@@ -1,10 +1,16 @@
+import logging
 import datetime
 import re
 import json
 from urllib.parse import quote
 from Crypto.Util import Padding
 from html import unescape
-from typing import Union
+from typing import Union, List
+
+
+
+log = logging.getLogger(__name__)
+log.setLevel(logging.DEBUG)
 
 
 def _get_l(d): return d['L']
@@ -121,16 +127,18 @@ class Period:
     @property
     def grades(self):
         """Get grades from the period."""
-        json_data = {'donnees': {'Periode': {'N': self.id, 'L': self.name}}, "_Signature_": {"onglet": 198}}
-        response = self._client.communication.post('DernieresNotes', json_data)
+        json_data = {'Periode': {'N': self.id, 'L': self.name}}
+        response = self._client.post('DernieresNotes', 198, json_data)
+        log.debug(response)
         grades = response['donneesSec']['donnees']['listeDevoirs']['V']
         return [Grade(g) for g in grades]
 
     @property
     def averages(self):
         """Get averages from the period."""
-        json_data = {'donnees': {'Periode': {'N': self.id, 'L': self.name}}, "_Signature_": {"onglet": 198}}
-        response = self._client.communication.post('DernieresNotes', json_data)
+
+        json_data = {'Periode': {'N': self.id, 'L': self.name}}
+        response = self._client.post('DernieresNotes', 198, json_data)
         crs = response['donneesSec']['donnees']['listeServices']['V']
         return [Average(c) for c in crs]
 
@@ -138,8 +146,8 @@ class Period:
     def overall_average(self):
         """Get overall average from the period. If the period average is not provided by pronote, then it's calculated.
         Calculation may not be the same as the actual average. (max difference 0.01)"""
-        json_data = {'donnees': {'Periode': {'N': self.id, 'L': self.name}}, "_Signature_": {"onglet": 198}}
-        response = self._client.communication.post('DernieresNotes', json_data)
+        json_data = {'Periode': {'N': self.id, 'L': self.name}}
+        response = self._client.post('DernieresNotes', 198, json_data)
         average = response['donneesSec']['donnees'].get('moyGenerale')
         if average:
             average = average['V']
@@ -164,6 +172,13 @@ class Period:
             average = -1
         return average
 
+    @property
+    def evaluations(self):
+        json_data = {'Periode': {'N': self.id, 'L': self.name, 'G': 2}}
+        response = self._client.post('DernieresEvaluations', 201, json_data)
+        evaluations = response['donneesSec']['donnees']['listeEvaluations']['V']
+        return [Evaluation(e) for e in evaluations]
+
 
 class Average:
     """
@@ -226,7 +241,7 @@ class Grade:
         the maximum amount of points
     default_out_of : str
         the default maximum amount of points
-    date : str
+    date : datetime.date
         the date on which the grade was given
     subject : pronotepy.dataClasses.Subject
         the subject in which the grade was given
@@ -240,19 +255,22 @@ class Grade:
         the lowest grade of the test
     coefficient : str
         the coefficient of the grade
+    comment : str
+        the comment on the grade description
     """
 
     id: str
     grade: str
     out_of: str
     default_out_of: str
-    date: str
+    date: datetime.date
     subject: Subject
     period: Period
     average: str
     max: str
     min: str
     coefficient: str
+    comment: str
 
     attribute_guide = {
         "N": ("id", str),
@@ -265,7 +283,7 @@ class Grade:
         "moyenne,V": ("average", Util.grade_parse),
         "noteMax,V": ("max", Util.grade_parse),
         "noteMin,V": ("min", Util.grade_parse),
-        "coefficient": ("coefficient", int),
+        "coefficient": ("coefficient", str),
         "commentaire": ("comment", str)
     }
 
@@ -310,7 +328,10 @@ class Lesson:
     start : str
         starting time of the lesson
     group_name : str
-        Name of the group."""
+        Name of the group.
+    exempted : bool
+        Specifies if the student's presence is exempt.
+    """
 
     id: str
     subject: Union[Subject, None]
@@ -322,9 +343,11 @@ class Lesson:
     outing: bool
     start: datetime.datetime
     group_name: str
+    exempted: bool
 
     __slots__ = ['id', 'subject', 'teacher_name', 'classroom', 'start',
-                 'canceled', 'status', 'background_color', 'detention', 'end', 'outing', 'group_name', 'student_class',
+                 'canceled', 'status', 'background_color', 'detention',
+                 'end', 'outing', 'group_name', 'student_class', 'exempted',
                  '_client', '_content']
     attribute_guide = {
         'DateDuCours,V': ('start', lambda d: datetime.datetime.strptime(d, '%d/%m/%Y %H:%M:%S')),
@@ -333,8 +356,8 @@ class Lesson:
         'Statut': ('status', str),
         'CouleurFond': ('background_color', str),
         'estRetenue': ('detention', bool),
-        'duree': ('end', int),
-        'estSortiePedagogique': ('outing', bool)
+        'estSortiePedagogique': ('outing', bool),
+        'dispenseEleve': ('exempted', bool)
     }
     transformers = {
         16: ('subject', Subject),
@@ -349,8 +372,20 @@ class Lesson:
         prepared_json = Util.prepare_json(self.__class__, parsed_json)
         for key in prepared_json:
             self.__setattr__(key, prepared_json[key])
-        if self.end:
-            self.end = self.start + client.one_hour_duration * self.end
+
+        # get correct ending time
+        # Pronote gives us the place where the hour should be in a week, when we modulo that with the amount of
+        # hours in a day we can get the "place" when the hour starts. Then we just add the duration (and substract 1)
+        end_place = parsed_json['place'] % (len(
+            client.func_options['donneesSec']['donnees']['General']['ListeHeuresFin']['V'])-1) + parsed_json['duree']-1
+
+        # With the end "place" now known we can look up the ending time in func_options
+        self.end = None
+        for end_time in client.func_options['donneesSec']['donnees']['General']['ListeHeuresFin']['V']:
+            if end_time['G'] == end_place:
+                end_time = datetime.datetime.strptime(end_time['L'], "%Hh%M").time()
+                self.end = self.start.replace(hour=end_time.hour, minute=end_time.minute)
+
         self.teacher_name = self.classroom = self.group_name = self.student_class = ''
         self.subject = None
         if 'ListeContenus' in parsed_json:
@@ -378,11 +413,11 @@ class Lesson:
         if self._content:
             return self._content
         week = self._client.get_week(self.start.date())
-        data = {"_Signature_": {"onglet": 89}, "donnees": {"domaine": {"_T": 8, "V": f"[{week}..{week}]"}}}
-        response = self._client.communication.post('PageCahierDeTexte', data)
+        data = {"domaine": {"_T": 8, "V": f"[{week}..{week}]"}}
+        response = self._client.post('PageCahierDeTexte', 89, data)
         contents = {}
         for lesson in response['donneesSec']['donnees']['ListeCahierDeTextes']['V']:
-            if lesson['cours']['V']['N'] == self.id:
+            if lesson['cours']['V']['N'] == self.id and lesson['listeContenus']['V']:
                 contents = lesson['listeContenus']['V'][0]
                 break
         if not contents:
@@ -546,10 +581,10 @@ class Homework:
         status : bool
             The status to which to change
         """
-        data = {'_Signature_': {'onglet': 88}, 'donnees': {'listeTAF': [
+        data = {'listeTAF': [
             {'N': self.id, 'TAFFait': status}
-        ]}}
-        if self._client.communication.post("SaisieTAFFaitEleve", data):
+        ]}
+        if self._client.post("SaisieTAFFaitEleve", 88, data):
             self.done = status
 
     @property
@@ -601,12 +636,11 @@ class Message:
     @property
     def content(self):
         """Returns the content of the message"""
-        data = {'donnees': {'message': {'N': self.id},
-                            'marquerCommeLu': False,
-                            'estNonPossede': False,
-                            'listePossessionsMessages': self._listePM},
-                '_Signature_': {'onglet': 131}}
-        resp = self._client.communication.post('ListeMessages', data)
+        data = {'message': {'N': self.id},
+                'marquerCommeLu': False,
+                'estNonPossede': False,
+                'listePossessionsMessages': self._listePM}
+        resp = self._client.post('ListeMessages', 131, data)
         for m in resp['donneesSec']['donnees']['listeMessages']['V']:
             if m['N'] == self.id:
                 if type(m['contenu']) == dict:
@@ -616,6 +650,171 @@ class Message:
         return None
 
 
+class ClientInfo:
+    """
+    Contains info for a resource (a client).
+
+    Attributes
+    ----------
+    id: str
+      id of the client (used internally)
+    name: str
+      name of the client
+    delegue: List[str]
+      list of classes of which the user is a delegue of
+    class_name: str
+      name of the student's class
+    raw_resource: dict
+      Raw json defining the resource
+    """
+
+    id: str
+
+    __slots__ = ['id', 'raw_resource']
+
+    def __init__(self, json_):
+        self.id = json_['N']
+        self.raw_resource = json_
+
+    @property
+    def name(self):
+        return self.raw_resource['L']
+
+    @property
+    def delegue(self):
+        if self.raw_resource['estDelegue']:
+            return [class_['L'] for class_ in self.raw_resource['listeClassesDelegue']['V']]
+        else:
+            return []
+
+    @property
+    def class_name(self):
+        return self.raw_resource.get('classeDEleve', {}).get('L', '')
+
+    @property
+    def establishment(self):
+        return self.raw_resource.get('Etablissement', {'V': {'L': ""}})['V']['L']
+
+
+class Acquisition:
+    """
+    Contains acquisition info for an evaluation.
+
+    Attributes
+    ----------
+    order: int
+        Telling the order in which the acquisition is. The list of acquisitions is already sorted by this.
+    level: str
+        the level achieved for this acquisition
+    id: int
+        id, used internally
+    abbreviation: str
+        abbreviation for the level achieved
+    coefficient: int
+        coefficient
+    domain: str
+        domain in which the acquisition is
+    domain_id: str
+    name: str
+        name (description) of the acquisition
+    name_id: str
+    pillar: str
+    pillar_id: str
+    pillar_prefix: str
+    """
+    order: int
+    level: str
+    id: str
+    abbreviation: str
+    coefficient: int
+    domain: str
+    domain_id: str
+    name: str
+    name_id: str
+    pillar: str
+    pillar_id: str
+    pillar_prefix: str
+
+    attribute_guide = {
+        'L': ('level', str),
+        'N': ('id', str),
+        'abbreviation': ('abbreviation', str),
+        'coefficient': ('coefficient', int),
+        'domaine,V,L': ('domain', str),
+        'domaine,V,N': ('domain_id', str),
+        'item,V,L': ('name', str),
+        'item,V,N': ('name_id', str),
+        'ordre': ('order', int),
+        'pilier,V,L': ('pillar', str),
+        'pilier,V,N': ('pillar_id', str),
+        'pilier,V,strPrefixes': ('pillar_prefix', str)
+    }
+
+    __slots__ = [
+        "order", "level", "id", "abbreviation", "coefficient", "domain", "domain_id", "name", "name_id", "pillar",
+        "pillar_id", "pillar_prefix"]
+
+    def __init__(self, json_):
+        prepared_json = Util.prepare_json(self.__class__, json_)
+        for key in prepared_json:
+            self.__setattr__(key, prepared_json[key])
+
+
+class Evaluation:
+    """
+    Data class for an evaluation.
+
+    Attributes
+    ----------
+    name: str
+    id: str
+    domain: str
+    teacher: str
+        the teacher who issued the evaluation
+    coefficient: int
+    description: str
+    subject: pronotepy.dataClasses.Subject
+    paliers: List[str]
+    acquisitions: List[pronotepy.dataClasses.Acquisition]
+    date: datetime.date
+    """
+    name: str
+    name: str
+    id: str
+    domain: str
+    teacher: str
+    coefficient: int
+    description: str
+    subject: Subject
+    paliers: List[str]
+    acquisitions: List[Acquisition]
+    date: datetime.date
+
+    __slots__ = [
+        "name", "name", "id", "domain", "teacher", "coefficient", "description", "subject", "paliers", "acquisitions",
+        "date"]
+
+    attribute_guide = {
+        'L': ('name', str),
+        'N': ('id', str),
+        'domaine,V,L': ('domain', str),
+        'individu,V,L': ('teacher', str),
+        'coefficient': ('coefficient', int),
+        'descriptif': ('description', str),
+        'matiere,V': ('subject', Subject),
+        'listePaliers,V': ('paliers', lambda x: [_get_l(y) for y in x]),
+        # Can we just appreciate the readability of this?
+        'listeNiveauxDAcquisitions,V': (
+            'acquisitions', lambda x: sorted([Acquisition(y) for y in x], key=lambda z: z.order)),
+        "date,V": ("date", lambda d: datetime.datetime.strptime(d, '%d/%m/%Y').date()),
+    }
+
+    def __init__(self, json_):
+        prepared_json = Util.prepare_json(self.__class__, json_)
+        for key in prepared_json:
+            self.__setattr__(key, prepared_json[key])
+
+
 class DataError(Exception):
     """
     Base exception for any errors made by creating or manipulating data classes.
