@@ -6,7 +6,7 @@ import secrets
 import threading
 import zlib
 from time import time, sleep
-from typing import List, Callable, Optional
+from typing import List, Callable, Optional, Union
 
 import requests
 from Crypto.Cipher import AES, PKCS1_v1_5
@@ -41,13 +41,14 @@ class Client(object):
 
     Attributes
     ----------
-    start_day : datetime.datetime
+    start_day : datetime.date
         The first day of the school year
     week : int
         The current week of the school year
     logged_in : bool
         If the user is successfully logged in
-
+    info: dataClasses.ClientInfo
+        Provides information about the current client. Name etc...
     """
 
     def __init__(self, pronote_url, username: str = '', password: str = '', ent: Optional[Callable] = None):
@@ -76,20 +77,13 @@ class Client(object):
         # some other attribute creation
         self._last_ping = time()
 
-        self.parametres_utilisateur = self.auth_cookie = None
+        self.parametres_utilisateur = self.auth_cookie = self.info = None
 
         self.date = datetime.datetime.now()
         self.start_day = datetime.datetime.strptime(
-            self.func_options['donneesSec']['donnees']['General']['PremierLundi']['V'], '%d/%m/%Y')
+            self.func_options['donneesSec']['donnees']['General']['PremierLundi']['V'], '%d/%m/%Y').date()
         self.week = self.get_week(datetime.date.today())
 
-        # get the length of one hour
-        hour_start = datetime.datetime.strptime(
-            self.func_options['donneesSec']['donnees']['General']['ListeHeures']['V'][0]['L'], '%Hh%M')
-        hour_end = datetime.datetime.strptime(
-            self.func_options['donneesSec']['donnees']['General']['ListeHeuresFin']['V'][0]['L'], '%Hh%M')
-        self.one_hour_duration = hour_end - hour_start
-
         self.periods_ = self.periods
         self.logged_in = self._login()
         self._expired = False
@@ -103,7 +97,7 @@ class Client(object):
         bool
             True if logged in, False if not
         """
-        if self.ent is True:
+        if self.ent:
             self.username = self.attributes['e']
             self.password = self.attributes['f']
         # identification phase
@@ -111,14 +105,14 @@ class Client(object):
             "genreConnexion": 0,
             "genreEspace": int(self.attributes['a']),
             "identifiant": self.username,
-            "pourENT": self.ent,
+            "pourENT": True if self.ent else False,
             "enConnexionAuto": False,
             "demandeConnexionAuto": False,
             "demandeConnexionAppliMobile": False,
             "demandeConnexionAppliMobileJeton": False,
             "uuidAppliMobile": "",
             "loginTokenSAV": ""}
-        idr = self.communication.post("Identification", {'donnees': ident_json})
+        idr = self.post("Identification", data=ident_json)
         log.debug('indentification')
 
         # creating the authentification data
@@ -128,7 +122,7 @@ class Client(object):
         e.aes_set_iv(self.communication.encryption.aes_iv)
 
         # key gen
-        if self.ent is True:
+        if self.ent:
             motdepasse = SHA256.new(str(self.password).encode()).hexdigest().upper()
             e.aes_key = MD5.new(motdepasse.encode()).digest()
         else:
@@ -149,15 +143,17 @@ class Client(object):
 
         # send
         auth_json = {"connexion": 0, "challenge": ch, "espace": int(self.attributes['a'])}
-        auth_response = self.communication.post("Authentification", {'donnees': auth_json})
+        auth_response = self.post("Authentification", data=auth_json)
         if 'cle' in auth_response['donneesSec']['donnees']:
-            self.communication.after_auth(self.communication.last_response, auth_response, e.aes_key)
+            self.communication.after_auth(auth_response, e.aes_key)
             self.encryption.aes_key = e.aes_key
             log.info(f'successfully logged in as {self.username}')
 
             # getting listeOnglets separately because of pronote API change
-            self.parametres_utilisateur = self.communication.post('ParametresUtilisateur', {})
-            self.communication.authorized_onglets = _prepare_onglets(self.parametres_utilisateur['donneesSec']['donnees']['listeOnglets'])
+            self.parametres_utilisateur = self.post('ParametresUtilisateur')
+            self.info = dataClasses.ClientInfo(self.parametres_utilisateur['donneesSec']['donnees']['ressource'])
+            self.communication.authorized_onglets = _prepare_onglets(
+                self.parametres_utilisateur['donneesSec']['donnees']['listeOnglets'])
             log.info("got onglets data.")
             return True
         else:
@@ -165,7 +161,7 @@ class Client(object):
             return False
 
     def get_week(self, date: datetime.date):
-        return 1 + int((date - self.start_day.date()).days / 7)
+        return 1 + int((date - self.start_day).days / 7)
 
     def lessons(self, date_from: datetime.date, date_to: datetime.date = None) -> List[dataClasses.Lesson]:
         """
@@ -178,12 +174,11 @@ class Client(object):
         :param date_to: second date
         """
         user = self.parametres_utilisateur['donneesSec']['donnees']['ressource']
-        data = {"_Signature_": {"onglet": 16},
-                "donnees": {"ressource": user,
-                            "avecAbsencesEleve": False, "avecConseilDeClasse": True,
-                            "estEDTPermanence": False, "avecAbsencesRessource": True,
-                            "avecDisponibilites": True, "avecInfosPrefsGrille": True,
-                            "Ressource": user}}
+        data = {"ressource": user,
+                "avecAbsencesEleve": False, "avecConseilDeClasse": True,
+                "estEDTPermanence": False, "avecAbsencesRessource": True,
+                "avecDisponibilites": True, "avecInfosPrefsGrille": True,
+                "Ressource": user}
         output = []
 
         first_week = self.get_week(date_from)
@@ -192,9 +187,9 @@ class Client(object):
         last_week = self.get_week(date_to)
 
         # getting lessons for all the weeks.
-        for week in range(first_week, last_week+1):
-            data['donnees']['NumeroSemaine'] = data['donnees']['numeroSemaine'] = week
-            response = self.communication.post('PageEmploiDuTemps', data)
+        for week in range(first_week, last_week + 1):
+            data['NumeroSemaine'] = data['numeroSemaine'] = week
+            response = self.post('PageEmploiDuTemps', 16, data)
             l_list = response['donneesSec']['donnees']['ListeCours']
             for lesson in l_list:
                 output.append(dataClasses.Lesson(self, lesson))
@@ -220,8 +215,9 @@ class Client(object):
     @property
     def current_period(self) -> dataClasses.Period:
         """Get the current period."""
-        id_period = self.parametres_utilisateur['donneesSec']['donnees']['ressource']['listeOngletsPourPeriodes']['V'][0][
-            'periodeParDefaut']['V']['N']
+        id_period = \
+            self.parametres_utilisateur['donneesSec']['donnees']['ressource']['listeOngletsPourPeriodes']['V'][0][
+                'periodeParDefaut']['V']['N']
         return dataClasses.Util.get(self.periods_, id=id_period)[0]
 
     def homework(self, date_from: datetime.date, date_to: datetime.date = None) -> List[dataClasses.Homework]:
@@ -241,10 +237,10 @@ class Client(object):
         if not date_to:
             date_to = datetime.datetime.strptime(
                 self.func_options['donneesSec']['donnees']['General']['DerniereDate']['V'], '%d/%m/%Y').date()
-        json_data = {'donnees': {
-            'domaine': {'_T': 8, 'V': f"[{self.get_week(date_from)}..{self.get_week(date_to)}]"}},
-            '_Signature_': {'onglet': 88}}
-        response = self.communication.post('PageCahierDeTexte', json_data)
+        json_data = {
+            'domaine': {'_T': 8, 'V': f"[{self.get_week(date_from)}..{self.get_week(date_to)}]"}}
+
+        response = self.post('PageCahierDeTexte', 88, json_data)
         h_list = response['donneesSec']['donnees']['ListeTravauxAFaire']['V']
         out = []
         for h in h_list:
@@ -269,8 +265,7 @@ class Client(object):
         List[Messages]
             Messages
         """
-        messages = self.communication.post('ListeMessagerie', {'donnees': {'avecMessage': True, 'avecLu': True},
-                                                               '_Signature_': {'onglet': 131}})
+        messages = self.post('ListeMessagerie', 131, {'avecMessage': True, 'avecLu': True})
         return [dataClasses.Message(self, m) for m in messages['donneesSec']['donnees']['listeMessagerie']['V']
                 if not m.get('estUneDiscussion')]
 
@@ -302,12 +297,43 @@ class Client(object):
 
     def session_check(self) -> bool:
         """Checks if the session has expired and refreshes it if it had (returns bool signifying if it was expired)"""
-        self.communication.post('Presence', {'_Signature_': {'onglet': 7}})
+        self.post('Presence', 7, {})
         if self._expired:
             self._expired = False
             return True
         return False
 
+    def post(self, function_name: str, onglet: int = None, data: dict = None):
+        """
+        Preforms a raw post to the PRONOTE server. Adds signature, then passes it to _Communication.post
+
+        Parameters
+        ----------
+        function_name: str
+        onglet: int
+        data: dict
+
+        Returns
+        -------
+        Raw JSON
+        """
+        post_data = {}
+        if onglet:
+            post_data['_Signature_'] = {'onglet': onglet}
+        if data:
+            post_data['donnees'] = data
+
+        try:
+            return self.communication.post(function_name, post_data)
+        except PronoteAPIError as e:
+            if isinstance(e, ExpiredObject):
+                raise e
+
+            log.info(
+                f'Have you tried turning it off and on again? ERROR: {e.pronote_error_code} | {e.pronote_error_msg}')
+            self.refresh()
+            return self.communication.post(function_name, data, True)
+
 
 class _Communication(object):
     def __init__(self, site, cookies, client):
@@ -348,10 +374,11 @@ class _Communication(object):
         self.compress_requests = not self.attributes.get("sCoA", False)
 
         # we need to catch this exception. the iv was not yet set and we need to decrypt it with the correct iv.
-        initial_response = self.post('FonctionParametres', {'donnees': json_post}, decryption_change={'iv': MD5.new(self.encryption.aes_iv_temp).digest()})
+        initial_response = self.post('FonctionParametres', {'donnees': json_post},
+                                     decryption_change={'iv': MD5.new(self.encryption.aes_iv_temp).digest()})
         return self.attributes, initial_response
 
-    def post(self, function_name: str, data: dict, recursive: bool = False, decryption_change=None):
+    def post(self, function_name: str, data: dict, decryption_change=None):
         """
         Handler for all POST requests by the api to PRONOTE servers. Automatically provides all needed data for the
         verification of posts. Session id and order numbers are preserved.
@@ -402,14 +429,10 @@ class _Communication(object):
             r_json = response.json()
             if r_json["Erreur"]["G"] == 22:
                 raise ExpiredObject(error_messages.get(22))
-            if recursive:
-                raise PronoteAPIError(error_messages.get(r_json["Erreur"]["G"],
-                                                         f'Unknown error from pronote: {r_json["Erreur"]["G"]} | {r_json["Erreur"]["Titre"]}'))
-
-            log.info(
-                f'Have you tried turning it off and on again? ERROR: {r_json["Erreur"]["G"]} | {r_json["Erreur"]["Titre"]}')
-            self.client.refresh()
-            return self.client.communication.post(function_name, data, True)
+            raise PronoteAPIError(error_messages.get(r_json["Erreur"]["G"],
+                                                     f'Unknown error from pronote: {r_json["Erreur"]["G"]} | {r_json["Erreur"]["Titre"]}'),
+                                  pronote_error_code=r_json['Erreur']['G'],
+                                  pronote_error_msg=r_json['Erreur']['Titre'])
 
         # checking for decryption change
         if decryption_change:
@@ -436,14 +459,12 @@ class _Communication(object):
 
         return response_data
 
-    def after_auth(self, auth_response, data, auth_key):
+    def after_auth(self, data, auth_key):
         """
         Key change after the authentification was successful.
 
         Parameters
         ----------
-        auth_response
-            The authentification response from the server
         auth_key
             AES authentification key used to calculate the challenge (From password of the user)
         data
@@ -568,11 +589,108 @@ class _KeepAlive(threading.Thread):
         self.join()
 
 
+class ParentClient(Client):
+    """
+    A parent PRONOTE client.
+
+    Parameters
+    ----------
+    pronote_url : str
+        URL of the server
+    username : str
+    password : str
+    ent : Callable
+        Cookies for ENT connections
+
+    Attributes
+    ----------
+    start_day : datetime.datetime
+        The first day of the school year
+    week : int
+        The current week of the school year
+    logged_in : bool
+        If the user is successfully logged in
+    info: dataClasses.ClientInfo
+        Provides information about the current client. Name etc...
+    children: List[dataClasses.ClientInfo]
+        List of sub-clients representing all the children connected to the main parent account.
+    """
+
+    def __init__(self, pronote_url, username: str = '', password: str = '', ent: Optional[Callable] = None,
+                 child: str = None):
+        super().__init__(pronote_url, username, password, ent)
+        self.children = []
+        self._selected_child = None
+        for c in self.parametres_utilisateur['donneesSec']['donnees']['ressource']['listeRessources']:
+            self.children.append(dataClasses.ClientInfo(c))
+
+        if not self.children:
+            raise ChildNotFound('No children were found.')
+        self.set_child(child if child else self.children[0])
+
+    def set_child(self, child: Union[str, dataClasses.ClientInfo]) -> None:
+        """
+        Select a child
+
+        Parameters
+        ----------
+        child: Union[str, dataClasses.ClientInfo]
+            Name or ClientInfo of a child.
+        """
+        if type(child) == str:
+            c = dataClasses.Util.get(self.children, name=child)
+            c = c[0] if c else None
+        else:
+            c = child
+
+        if not c:
+            raise ChildNotFound(f"A child with the name {child} was not found.")
+
+        self._selected_child = c
+        self.parametres_utilisateur['donneesSec']['donnees']['ressource'] = c.raw_resource
+
+    def post(self, function_name: str, onglet: int = None, data: dict = None):
+        """
+        Preforms a raw post to the PRONOTE server. Adds signature, then passes it to _Communication.post
+
+        Parameters
+        ----------
+        function_name: str
+        onglet: int
+        data: dict
+
+        Returns
+        -------
+        Raw JSON
+        """
+        post_data = {}
+        if onglet:
+            post_data['_Signature_'] = {'onglet': onglet, 'membre': {'N': self._selected_child.id, 'G': 4}}
+
+        if data:
+            post_data['donnees'] = data
+
+        try:
+            return self.communication.post(function_name, post_data)
+        except PronoteAPIError as e:
+            if isinstance(e, ExpiredObject):
+                raise e
+
+            log.info(
+                f'Have you tried turning it off and on again? ERROR: {e.pronote_error_code} | {e.pronote_error_msg}')
+            self.refresh()
+            return self.communication.post(function_name, data, True)
+
+
 class PronoteAPIError(Exception):
     """
     Base exception for any pronote api errors
     """
-    pass
+
+    def __init__(self, *args: object, pronote_error_code: int = None, pronote_error_msg: str = None) -> None:
+        super().__init__(*args)
+        self.pronote_error_code = pronote_error_code
+        self.pronote_error_msg = pronote_error_msg
 
 
 class CryptoError(PronoteAPIError):
@@ -583,3 +701,7 @@ class CryptoError(PronoteAPIError):
 class ExpiredObject(PronoteAPIError):
     """Raised when pronote returns error 22. (unknown object reference)"""
     pass
+
+
+class ChildNotFound(PronoteAPIError):
+    """Child with this name was not found."""
